﻿namespace AtomicTorch.CBND.CoreMod.Characters.Mobs
{
    using System;
    using AtomicTorch.GameEngine.Common.Primitives;
    using AtomicTorch.CBND.CoreMod.CharacterSkeletons;
    using AtomicTorch.CBND.CoreMod.Items.Food;
    using AtomicTorch.CBND.CoreMod.Items.Generic;
    using AtomicTorch.CBND.CoreMod.Items.Weapons.MobWeapons;
    using AtomicTorch.CBND.CoreMod.Skills;
    using AtomicTorch.CBND.CoreMod.SoundPresets;
    using AtomicTorch.CBND.CoreMod.Stats;
    using AtomicTorch.CBND.CoreMod.Systems.Droplists;

    public class MobCamel : ProtoCharacterMob
    {
        public override float CharacterWorldHeight => 1.9f;

        public override double MobKillExperienceMultiplier => 0.1;

        public override string Name => "Camel";

        public override ObjectSoundMaterial ObjectSoundMaterial => ObjectSoundMaterial.SoftTissues;

        public override double StatDefaultHealthMax => 150;

        public override double StatMoveSpeed => 1.5;

        protected override void FillDefaultEffects(Effects effects)
        {
            base.FillDefaultEffects(effects);

            effects.AddValue(this, StatName.DefenseImpact, 0.4);
        }

        protected override void PrepareProtoCharacterMob(
            out ProtoCharacterSkeleton skeleton,
            ref double scale,
            DropItemsList lootDroplist)
        {
            skeleton = GetProtoEntity<SkeletonCamel>();

            // primary loot
            lootDroplist
                .Add<ItemMeatRaw>(count: 1)
                .Add<ItemLeather>(count: 1, countRandom: 1);

            // random loot
            lootDroplist.Add(nestedList: new DropItemsList(outputs: 1)
                                         .Add<ItemAnimalFat>(count: 1, countRandom: 1)
                                         .Add<ItemBones>(count: 1,     countRandom: 1));

            // extra loot
            lootDroplist.Add(condition: SkillHunting.ServerRollExtraLoot,
                             nestedList: new DropItemsList(outputs: 2)
                                         .Add<ItemMeatRaw>(count: 1)
                                         .Add<ItemLeather>(count: 1)
                                         .Add<ItemAnimalFat>(count: 1)
                                         .Add<ItemBones>(count: 1));
        }

        protected override void ServerInitializeCharacterMob(ServerInitializeData data)
        {
            base.ServerInitializeCharacterMob(data);

            var weaponProto = GetProtoEntity<ItemWeaponGenericAnimalMedium>();
            data.PrivateState.WeaponState.SetWeaponProtoOnly(weaponProto);
            data.PublicState.SetCurrentWeaponProtoOnly(weaponProto);
        }

        protected override void ServerUpdateMob(ServerUpdateData data)
        {
            var character = data.GameObject;

            var movementDirection = Vector2F.Zero;
            var rotationAngleRad = double.NaN;
			
            var characterPrivateState = character.GetPrivateState<CharacterMobPrivateState>();
            var enemyCharacter = ServerCharacterAiHelper.GetClosestPlayer(character);

            var distanceToEnemy = double.NaN;
            var directionToEnemy = Vector2F.Zero;

            if (enemyCharacter == null)
            {
                distanceToEnemy = double.NaN;
                directionToEnemy = Vector2F.Zero;
                return;
            }

            var deltaPos = enemyCharacter.Position - character.Position;
            distanceToEnemy = deltaPos.Length;
            directionToEnemy = (Vector2F)deltaPos;

            movementDirection = distanceToEnemy < 3
                                    || distanceToEnemy > 6
                                        ? Vector2F.Zero // too close or too far
                                        : directionToEnemy;

            rotationAngleRad = character.GetPublicState<CharacterMobPublicState>()
                                           .AppliedInput
                                           .RotationAngleRad;
										   
            if (directionToEnemy != Vector2F.Zero)
            {
                rotationAngleRad = Math.Abs(
                    Math.Atan2(directionToEnemy.Y, directionToEnemy.X) + 2 * Math.PI);
            }

            this.ServerSetMobInput(character, movementDirection, rotationAngleRad);
			
			/*
            var character = data.GameObject;

			var enemyCharacter = ServerCharacterAiHelper.GetClosestPlayer(character);
		    var deltaPos = enemyCharacter.Position - character.Position;
            var distanceToEnemy = deltaPos.Length;
            var directionToEnemy = (Vector2F)deltaPos;

            Vector2F movementDirection = distanceToEnemy < 3
                                    || distanceToEnemy > 6
                                        ? Vector2F.Zero // too close or too far
                                        : directionToEnemy;

            var rotationAngleRad = this.SharedGetRotationAngleRad(character);
            if (directionToEnemy != Vector2F.Zero)
            {
                rotationAngleRad = Math.Abs(
                    Math.Atan2(directionToEnemy.Y, directionToEnemy.X) + 2 * Math.PI);
            }

			data.PrivateState.WeaponState.SetInputIsFiring(false); //passive mob

            this.ServerSetMobInput(character, movementDirection, rotationAngleRad);
			*/
        }
    }
}